@@ -13,10 +13,11 @@
  * for 0.87: output "charset iso-8859-x" so that loadkeys
  *      can handle the output of dumpkeys -c
  * for 0.88: handle sparse keymaps
- *
  * for 0.94: support alt_is_meta
+ * for 0.96: option -1
  */
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <getopt.h>
@@ -27,6 +28,7 @@
 #include <string.h>
 #include <errno.h>
 #include "ksyms.h"
+#include "modifiers.h"
 
 #ifndef KT_LETTER
 #define KT_LETTER KT_LATIN
@@ -36,7 +38,7 @@
 #define MAX_NR_KEYMAPS NR_KEYMAPS
 #endif
 
-#define VERSION "0.94"
+#define VERSION "0.96"
 extern int getfd();
 static int fd;
 
@@ -120,21 +122,21 @@ void print_keysym(int code, char numeric) {
 	t = KTYP(code);
 	v = KVAL(code);
 	if (t > KT_LETTER) {
-	        printf("U+%04x          ", code ^ 0xf000);
+		printf("U+%04x          ", code ^ 0xf000);
 		return;
 	}
 	if (t == KT_LETTER) {
-	        t = KT_LATIN;
-		    printf("+");
+		t = KT_LATIN;
+		printf("+");
 	}
 	if (!numeric && t < syms_size && v < syms[t].size &&
-		 (p = syms[t].table[v])[0])
-	     printf("%-16s", p);
+	    (p = syms[t].table[v])[0])
+		printf("%-16s", p);
 	else if (!numeric && t == KT_META && v < 128 && v < syms[0].size &&
 		 (p = syms[0].table[v])[0])
-	     printf("Meta_%-11s", p);
+		printf("Meta_%-11s", p);
 	else
-	     printf("0x%04x          ", code);
+		printf("0x%04x          ", code);
 }
 
 char valid_type(int t) {
@@ -175,8 +177,8 @@ int maxval[NR_TYPES];
    be an option. Use locale? */
 void
 outchar (unsigned char c) {
-        printf("'");
-        printf((c == '\'' || c == '\\') ? "\\%c"
+	printf("'");
+	printf((c == '\'' || c == '\\') ? "\\%c"
 	       : (isgraph(c) || c == ' ' || c >= 0200) ? "%c"
 	       : "\\%03o", c);
 	printf("'");
@@ -205,7 +207,7 @@ void dump_diacs(void) {
 
 	get_diacs();
 	for (i = 0; i < kd.kb_cnt; i++) {
-	        printf("compose ");
+		printf("compose ");
 		outchar(kd.kbdiacr[i].diacr);
 		printf(" ");
 		outchar(kd.kbdiacr[i].base);
@@ -285,36 +287,49 @@ void dump_symbols(void) {
 }
 
 void print_mod(int x) {
-        int t;
+	int t;
 
+	if (!x)
+		printf("plain\t");
+	else
 	for (t = 0; t < sizeof(modifiers)/sizeof(modifiers[0]); t++)
 	  if (x & (1 << modifiers[t].bit))
 	    printf("%s\t", modifiers[t].name);
 }
 
 void print_bind(int bufj, int i, int j, char numeric) {
-	printf("\t");
+	if(j)
+	    printf("\t");
 	print_mod(j);
 	printf("keycode %3d =", i);
 	print_keysym(bufj, numeric);
 	printf("\n");
 }
 
-void dump_keys(char full_table, char numeric) {
+#define DEFAULT		0
+#define FULL_TABLE	1	/* one line for each keycode */
+#define SEPARATE_LINES	2	/* one line for each (modifier,keycode) pair */
+#define	UNTIL_HOLE	3	/* one line for each keycode, until 1st hole */
+
+void dump_keys(char table_shape, char numeric) {
 	int i, j, k;
 	int buf[MAX_NR_KEYMAPS];
 	int isletter, islatin, isasexpected;
 	int typ, val;
 	int alt_is_meta = 0;
+	int zapped[MAX_NR_KEYMAPS];
 
 	get_keymaps();
 	print_keymaps();
 	if (!keymapnr)
 	  return;
 
+	if (table_shape == FULL_TABLE || table_shape == SEPARATE_LINES)
+	  goto no_shorthands;
+
 	/* first pass: determine whether to set alt_is_meta */
 	for (j = 0; j < MAX_NR_KEYMAPS; j++) {
-	     int ja = (j | (1 << KG_ALT));
+	     int ja = (j | M_ALT);
 	     if (j != ja && keymap_index[j] >= 0 && keymap_index[ja] >= 0)
 		  for (i = 1; i < NR_KEYS; i++) {
 		       int buf0, buf1, type;
@@ -341,10 +356,26 @@ void dump_keys(char full_table, char numeric) {
 	printf("alt_is_meta\n");
 not_alt_is_meta:
 
+no_shorthands:
 	for (i = 1; i < NR_KEYS; i++) {
 	    for (j = 0; j < keymapnr; j++)
 	      buf[j] = get_bind(i, good_keymap[j]);
 
+	    if (table_shape == FULL_TABLE) {
+		printf("keycode %3d =", i);
+		for (j = 0; j < keymapnr; j++)
+		  print_keysym(buf[j], numeric);
+		printf("\n");
+		continue;
+	    }
+
+	    if (table_shape == SEPARATE_LINES) {
+		for (j = 0; j < keymapnr; j++)
+		  print_bind(buf[j], i, good_keymap[j], numeric);
+		printf("\n");
+		continue;
+	    }
+
 	    typ = KTYP(buf[0]);
 	    val = KVAL(buf[0]);
 	    islatin = (typ == KT_LATIN || typ == KT_LETTER);
@@ -373,11 +404,13 @@ not_alt_is_meta:
 	  unexpected:
 
 	    /* wipe out predictable meta bindings */
+	    for (j = 0; j < keymapnr; j++)
+		    zapped[j] = 0;
 	    if (alt_is_meta) {
 		 for(j = 0; j < keymapnr; j++) {
 		      int ka, ja, typ;
 		      k = good_keymap[j];
-		      ka = (k | (1 << KG_ALT));
+		      ka = (k | M_ALT);
 		      ja = keymap_index[ka];
 		      if (k != ka && ja >= 0
 		       && ((typ=KTYP(buf[j])) == KT_LATIN || typ == KT_LETTER)
@@ -385,54 +418,45 @@ not_alt_is_meta:
 			   if (buf[ja] != K(KT_META, KVAL(buf[j])))
 				fprintf(stderr, "impossible: not meta?\n");
 			   buf[ja] = K_HOLE;
+			   zapped[ja] = 1;
 		      }
 		 }
 	    }
 
 	    printf("keycode %3d =", i);
-	    if (full_table) {
-		for (j = 0; j < keymapnr; j++)
-		  print_keysym(buf[j], numeric);
-		printf("\n");
-	    } else
 	    if (isasexpected) {
 		/* print only a single entry */
 		/* suppress the + for ordinary a-zA-Z */
 		print_keysym(K(KT_LATIN, val), numeric);
 		printf("\n");
-	    } else if (!islatin) {
+	    } else {
 		/* choose between single entry line followed by exceptions,
 		   and long line followed by exceptions; avoid VoidSymbol */
 		int bad = 0;
 		int count = 0;
-		for(j = 1; j < keymapnr; j++) {
+		for(j = 1; j < keymapnr; j++) if (!zapped[j]) {
 		    if (buf[j] != buf[0])
 		      bad++;
 		    if (buf[j] != K_HOLE)
 		      count++;
 		}
-		if (bad <= count) {
+		if (bad <= count && bad < keymapnr-1) {
 		    if (buf[0] != K_HOLE)
 		      print_keysym(buf[0], numeric);
 		    printf("\n");
 		    for (j = 1; j < keymapnr; j++)
-		      if (buf[j] != buf[0])
+		      if (buf[j] != buf[0] && !zapped[j])
 			print_bind(buf[j], i, good_keymap[j], numeric);
 		} else {
-		    for (j = 0; j < keymapnr && buf[j] != K_HOLE; j++)
+		    for (j = 0; j < keymapnr && buf[j] != K_HOLE &&
+				 (j == 0 || table_shape != UNTIL_HOLE ||
+				  good_keymap[j] == good_keymap[j-1]+1); j++)
 		      print_keysym(buf[j], numeric);
 		    printf("\n");
 		    for ( ; j < keymapnr; j++)
 		      if (buf[j] != K_HOLE)
 			print_bind(buf[j], i, good_keymap[j], numeric);
 		}
-	    } else {
-		for (j = 0; j < keymapnr && buf[j] != K_HOLE; j++)
-		  print_keysym(buf[j], numeric);
-		printf("\n");
-		for ( ; j < keymapnr; j++)
-		  if (buf[j] != K_HOLE)
-		    print_bind(buf[j], i, good_keymap[j], numeric);
 	    }
 	}
 }
@@ -452,11 +476,11 @@ void dump_funcs(void) {
 		    exit(1);
 		}
 		if (!fbuf.kb_string[0])
-		        continue;
+			continue;
 		printf("string %s = \"", syms[KT_FN].table[i]);
 		for (p = fbuf.kb_string; *p; p++) {
-		        if (*p == '"' || *p == '\\') {
-			        putchar('\\'); putchar(*p);
+			if (*p == '"' || *p == '\\') {
+				putchar('\\'); putchar(*p);
 			} else if (isgraph(*p) || *p == ' ')
 				putchar(*p);
 			else
@@ -474,20 +498,21 @@ usage: dumpkeys [options...]
 
 valid options are:
 
-	-h --help	display this help text
-	-i --short-info	display information about keyboard driver
-	-l --long-info	display above and symbols known to loadkeys
-	-n --numeric	display keytable in hexadecimal notation
-	-f --full-table	don't use short-hand notations
-	   --funcs-only	display only the function key strings
-	   --keys-only	display only key bindings
-%s	-c --charset=iso-8859-{1,2,3,4,5,7,8}
+	-h --help	    display this help text
+	-i --short-info	    display information about keyboard driver
+	-l --long-info	    display above and symbols known to loadkeys
+	-n --numeric	    display keytable in hexadecimal notation
+	-f --full-table	    don't use short-hand notations, one row per keycode
+	-1 --separate-lines one line per (modifier,keycode) pair
+	   --funcs-only	    display only the function key strings
+	   --keys-only	    display only key bindings
+%s	-c --charset=iso-8859-{1,2,3,4,5,7,8,9}
 	             koi8-{r,u}		
-			interpret character action codes to be from the
-			specified character set
+			    interpret character action codes to be from the
+			    specified character set
 ",
 #ifdef KDGKBDIACR
-"           --compose-only  display only compose key combinations\n"
+"           --compose-only   display only compose key combinations\n"
 #else
 ""
 #endif
@@ -497,17 +522,19 @@ valid options are:
 
 int
 main (int argc, char *argv[]) {
-	const char *short_opts = "hilvsnfc:";
+	const char *short_opts = "hilvsnf1S:c:";
 	const struct option long_opts[] = {
 		{ "help",	no_argument,		NULL, 'h' },
 		{ "short-info",	no_argument,		NULL, 'i' },
 		{ "long-info",	no_argument,		NULL, 'l' },
 		{ "numeric",	no_argument,		NULL, 'n' },
 		{ "full-table",	no_argument,		NULL, 'f' },
+		{ "separate-lines",no_argument,		NULL, '1' },
+		{ "shape",	required_argument,	NULL, 'S' },
 		{ "funcs-only",	no_argument,		NULL, 't' },
 		{ "keys-only",	no_argument,		NULL, 'k' },
 #ifdef KDGKBDIACR
-		{ "compose-only",no_argument,           NULL, 'd' },
+		{ "compose-only",no_argument,		NULL, 'd' },
 #endif
 		{ "verbose",	no_argument,		NULL, 'v' },
 		{ "charset",	required_argument,	NULL, 'c' },
@@ -517,7 +544,7 @@ main (int argc, char *argv[]) {
 	char long_info = 0;
 	char short_info = 0;
 	char numeric = 0;
-	char full_table = 0;
+	char table_shape = 0;
 	char funcs_only = 0;
 	char keys_only = 0;
 #ifdef KDGKBDIACR
@@ -538,7 +565,13 @@ main (int argc, char *argv[]) {
 				numeric = 1;
 				break;
 			case 'f':
-				full_table = 1;
+				table_shape = FULL_TABLE;
+				break;
+			case '1':
+				table_shape = SEPARATE_LINES;
+				break;
+			case 'S':
+				table_shape = atoi(optarg);
 				break;
 			case 't':
 				funcs_only = 1;
@@ -581,13 +614,13 @@ main (int argc, char *argv[]) {
 	if (!diac_only) {
 #endif
 	    if (!funcs_only)
-		dump_keys(full_table, numeric);
+		dump_keys(table_shape, numeric);
 	    if (!keys_only)
 		dump_funcs();
 #ifdef KDGKBDIACR
 	}
 	if (!funcs_only && !keys_only)
-	        dump_diacs();
+		dump_diacs();
 #endif
 
 	exit(0);
