@@ -456,6 +456,18 @@ static char *iso_8859_8_syms[] = {
     "resh", "shin", "tav", "", "", "", "", ""
 };
 
+static char *iso_8859_9_syms[] = { /* latin-5 */
+	/* Identical to latin-1, but with the 6 symbols
+	   ETH, eth, THORN, thorn, Yacute, yacute replaced by
+	   Gbreve, gbreve, Scedilla, scedilla, Idotabove, dotlessi */
+	"Gbreve", "", "", "", "", "", "", "",         /* 0320-0327 */
+	"", "", "", "", "", "Idotabove", "Scedilla", "",
+	"", "", "", "", "", "", "", "",	              /* 0340-0347 */
+	"", "", "", "", "", "", "", "",	              /* 0350-0357 */
+	"gbreve", "", "", "", "", "", "", "",         /* 0360-0367 */
+	"", "", "", "", "", "dotlessi", "scedilla", ""
+};
+
 #include "cyrillic.syms.h"
 #include "ethiopic.syms.h"
 
@@ -541,7 +553,7 @@ static char *spec_syms[] = {
 	"SAK",
 	"Decr_Console",
 	"Incr_Console",
-	"Spawn_Console",
+	"KeyboardSignal",
 	"Bare_Num_Lock"
 };
 
@@ -759,7 +771,7 @@ synonyms[] = {
 	{ "AltLLock",		"Alt_Lock" },
 	{ "AltRLock",		"AltGr_Lock" },
 	{ "SCtrl",		"SControl" },
-	{ "KeyboardSignal",	"Spawn_Console" },
+	{ "Spawn_Console",	"KeyboardSignal" },
 /* the names of the Greek letters are spelled differently
    in the iso-8859-7 and the Unicode standards */
 	{ "lambda",             "lamda" },
@@ -771,7 +783,7 @@ synonyms[] = {
 /* diacriticals */
 	{ "tilde",		"asciitilde" },
 	{ "circumflex",		"asciicircum" },
-/* as dead_ogonek, dead_caron, dead_breve and dead_doubleacute doesn't exist
+/* as dead_ogonek, dead_caron, dead_breve and dead_doubleacute do not exist
  * (yet), I put also compose lines for use with respectively dead_cedilla,
  * dead_circumflex, dead_tilde and dead_tilde */
 	{ "dead_ogonek",        "dead_cedilla" },
@@ -815,6 +827,7 @@ struct cs {
     { "iso-8859-5", iso_8859_5_syms, 160 },
     { "iso-8859-7", iso_8859_7_syms, 160 },
     { "iso-8859-8", iso_8859_8_syms, 160 },
+    { "iso-8859-9", iso_8859_9_syms, 208 },
     { "koi8-r",     koi8_r_syms, 160 },
     { "koi8-u",     koi8_r_syms, 160 },
 /* I don't know the differences between koi8-r and koi8-u,
@@ -852,7 +865,7 @@ ksymtocode(char *s)
 	int i;
 	int j;
 	int keycode;
-	static warn = 0;
+	static int warn = 0;
 
 	if (!strncmp(s, "Meta_", 5)) {
 		keycode = ksymtocode(s+5);
@@ -873,21 +886,21 @@ ksymtocode(char *s)
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin2_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-2\n");
+				fprintf(stderr, "assuming iso-8859-2 %s\n", s);
 			return K(KT_LATIN, 160 + i);
 		}
 
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin3_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-3\n");
+				fprintf(stderr, "assuming iso-8859-3 %s\n", s);
 			return K(KT_LATIN, 160 + i);
 		}
 
 	for (i = 0; i < 256 - 160; i++)
 		if (!strcmp(s, latin4_syms[i])) {
 			if(!warn++)
-				fprintf(stderr, "assuming iso-8859-4\n");
+				fprintf(stderr, "assuming iso-8859-4 %s\n", s);
 			return K(KT_LATIN, 160 + i);
 		}
 
