@@ -1,4 +1,6 @@
-#define PSF_MAGIC      0x0436
+#define PSF_MAGIC1	0x36
+#define PSF_MAGIC2	0x04
+
 #define PSF_MODE512    0x01
 #define PSF_MODEHASTAB 0x02
 #define PSF_MAXMODE    0x03
@@ -6,7 +8,9 @@
 
 struct psf_header
 {
-  unsigned short magic;		/* Magic number */
+  unsigned char magic1, magic2;	/* Magic number */
   unsigned char mode;		/* PSF font mode */
   unsigned char charsize;	/* Character size */
 };
+
+#define PSF_MAGIC_OK(x)	((x).magic1 == PSF_MAGIC1 && (x).magic2 == PSF_MAGIC2)
