@@ -8,7 +8,7 @@
 #include <linux/keyboard.h>
 #include <sys/ioctl.h>
 
-#define VERSION "0.91"
+#define VERSION "0.96"
 
 int tmp;	/* for debugging */
 
@@ -37,6 +37,8 @@ void get_mode(void) {
 	    m = "XLATE"; break;
 	  case K_MEDIUMRAW:
 	    m = "MEDIUMRAW"; break;
+	  case K_UNICODE:
+	    m = "UNICODE"; break;
 	  default:
 	    m = "?UNKNOWN?"; break;
 	}
@@ -79,6 +81,7 @@ usage: showkey [options...]
 valid options are:
 
 	-h --help	display this help text
+	-a --ascii	display the decimal/octal/hex values of the keys
 	-s --scancodes	display only the raw scan-codes
 	-k --keycodes	display only the interpreted keycodes (default)
 ");
@@ -87,15 +90,17 @@ valid options are:
 
 int
 main (int argc, char *argv[]) {
-	const char *short_opts = "hsk";
+	const char *short_opts = "hask";
 	const struct option long_opts[] = {
 		{ "help",	no_argument, NULL, 'h' },
+		{ "ascii",	no_argument, NULL, 'a' },
 		{ "scancodes",	no_argument, NULL, 's' },
 		{ "keycodes",	no_argument, NULL, 'k' },
 		{ NULL, 0, NULL, 0 }
 	};
 	int c;
 	int show_keycodes = 1;
+	int print_ascii = 0;
 
 	struct termios new;
 	unsigned char buf[16];
@@ -110,6 +115,9 @@ main (int argc, char *argv[]) {
 			case 'k':
 				show_keycodes = 1;
 				break;
+			case 'a':
+				print_ascii = 1;
+				break;
 			case 'h':
 			case '?':
 				usage();
@@ -119,6 +127,45 @@ main (int argc, char *argv[]) {
 	if (optind < argc)
 		usage();
 
+	if (print_ascii) {
+		/* no mode and signal and timer stuff - just read stdin */
+	        fd = 0;
+
+		tmp = tcgetattr(fd, &old);
+		if (tmp)
+			printf("tcgetattr = %d\n", tmp);
+		tmp = tcgetattr(fd, &new);
+		if (tmp)
+			printf("tcgetattr = %d\n", tmp);
+
+		new.c_lflag &= ~ (ICANON | ISIG);
+		new.c_lflag |= (ECHO | ECHOCTL);
+		new.c_iflag = 0;
+		new.c_cc[VMIN] = 1;
+		new.c_cc[VTIME] = 0;
+
+		tmp = tcsetattr(fd, TCSAFLUSH, &new);
+		if (tmp)
+			printf("tcsetattr = %d\n", tmp);
+		printf("\nPress any keys - "
+		       "Ctrl-D will terminate this program\n\n");
+
+		while (1) {
+			n = read(fd, buf, 1);
+			if (n == 1)
+				printf(" \t%3d 0%03o 0x%02x\n",
+				       buf[0], buf[0], buf[0]);
+			if (n != 1 || buf[0] == 04)
+				break;
+		}
+
+		tmp = tcsetattr(fd, 0, &old);
+		if (tmp)
+                	printf("tcsetattr = %d\n", tmp);
+		exit(0);
+	}
+
+
 	fd = getfd();
 
 	/* the program terminates when there is no input for 10 secs */
